
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripMap()
{
	this.obj = {}
	this.objId = {}
	this.length = 0;

	this.add = function(page)
	{
		if (!page) return;
		if (!this.obj[page.getName()]) this.length++;
		this.obj[page.getName()] = page;
		this.objId[page.getId()] = page;
	}

	this.removeByName = function(pName)
	{
		if (!pName || !this.obj[pName]) return;
		this.objId[this.obj[pName].getId()] = null;
		this.obj[pName] = null;
		this.length--;
	}

	this.removeById = function(id)
	{
		if (!id || !this.objId[id]) return;
		this.obj[this.objId[id].getName()] = null;
		this.objId[id] = null;
		this.length--;
	}

	this.clone = function()
	{
		var map = new this.constructor();
		for (var p in this.obj) if (this.obj[p]) map.add(this.obj[p].clone());
		return map;
	}

	this.sortPages = function(reduce)
	{
		var aPage = [];
		for (var p in this.obj)
		{
			if (!this.obj[p]) continue;
			this.obj[p].whitelist.sortXPaths(reduce);
			this.obj[p].blacklist.sortXPaths(reduce);
			this.obj[p].styles.sortXPaths();
			this.obj[p].extension.sortXPaths();
			aPage.push(this.obj[p]);
		}
		aPage.sort(this.pageSort);
		this.obj = {}
		for (var i = 0; i < aPage.length; i++) this.obj[aPage[i].getName()] = aPage[i];
	}

	this.generateXml = function()
	{
		var r = "<?xml version=\"1.0\" encoding=\"" + yarip.CHARSET + "\"?>\n\n<!--\nDo not edit this file.\n\nIf you make changes to this file while the application is running,\nthe changes will be overwritten when the application exits.\n-->\n\n<yarip>\n";
		for (var p in this.obj) if (this.obj[p]) r += this.obj[p].generateXml();
		r += "</yarip>";
		return r;
	}

	this.generateCSS = function()
	{
		var r = "\n/* Do not edit this file.\n *\n * If you make changes to this file while the application is running,\n * the changes will be overwritten when the application exits.\n */\n\n";
		for (var p in this.obj) if (this.obj[p]) r += this.obj[p].generateCSS();
		return r;
	}

	this.pageSort = function(a, b)
	{
		var findDomainRegExp = /\b((((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5]))|(\[((([a-f\d]{1,4}:){6}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(::([a-f\d]{1,4}:){5}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(([a-f\d]{1,4})?::([a-f\d]{1,4}:){4}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|((([a-f\d]{1,4}:)?[a-f\d]{1,4})?::([a-f\d]{1,4}:){3}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,2})?[a-f\d]{1,4})?::([a-f\d]{1,4}:){2}([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,3})?[a-f\d]{1,4})?::([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,4})?[a-f\d]{1,4})?::([a-f\d]{1,4}:[a-f\d]{1,4})|((([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])\.){3}([1-9]?\d|1\d{2}|2[0-4]\d|25[0-5])))|(((([a-f\d]{1,4}:){1,5})?[a-f\d]{1,4})?::[a-f\d]{1,4})|(((([a-f\d]{1,4}:){1,6})?[a-f\d]{1,4})?::))\]))|([a-z\d]([\w-.~!$&'()*+,;=]|%[a-f\d]{2})+\.(ac|ad|aero|ae|af|ag|ai|al|am|an|ao|aq|arpa|ar|asia|as|at|au|aw|ax|az|ba|bb|bd|be|bf|bg|bh|biz|bi|bj|bm|bn|bo|br|bs|bt|bv|bw|by|bz|cat|ca|cc|cd|cf|cg|ch|ci|ck|cl|cm|cn|coop|com|co|cr|cu|cv|cx|cy|cz|de|dj|dk|dm|do|dz|ec|edu|ee|eg|er|es|et|eu|fi|fj|fk|fm|fo|fr|ga|gb|gd|ge|gf|gg|gh|gi|gl|gm|gn|gov|gp|gq|gr|gs|gt|gu|gw|gy|hk|hm|hn|hr|ht|hu|id|ie|il|im|info|int|in|io|iq|ir|is|it|je|jm|jobs|jo|jp|ke|kg|kh|ki|km|kn|kp|kr|kw|ky|kz|la|lb|lc|li|lk|lr|ls|lt|lu|lv|ly|ma|mc|md|me|mg|mh|mil|mk|ml|mm|mn|mobi|mo|mp|mq|mr|ms|mt|museum|mu|mv|mw|mx|my|mz|name|na|nc|net|ne|nf|ng|ni|nl|no|np|nr|nu|nz|om|org|pa|pe|pf|pg|ph|pk|pl|pm|pn|pro|pr|ps|pt|pw|py|qa|re|ro|rs|ru|rw|sa|sb|sc|sd|se|sg|sh|si|sj|sk|sl|sm|sn|so|sr|st|su|sv|sy|sz|tc|td|tel|tf|tg|th|tj|tk|tl|tm|tn|to|tp|travel|tr|tt|tv|tw|tz|ua|ug|uk|um|us|uy|uz|va|vc|ve|vg|vi|vn|vu|wf|ws|xn--0zwm56d|xn--11b5bs3a9aj6g|xn--80akhbyknj4f|xn--9t4b11yi5a|xn--deba0ad|xn--g6w251d|xn--hgbk6aj7f53bba|xn--hlcj6aya9esc7a|xn--jxalpdlp|xn--kgbechtv|xn--zckzah|ye|yt|yu|za|zm|zw)))\b/i;
		var aArr = a.name.match(findDomainRegExp)[0].split('.').reverse();
		var bArr = b.name.match(findDomainRegExp)[0].split('.').reverse();
		for (var i = 1; i >= 0; i--)
		{
			if (aArr[i] < bArr[i]) return -1;
			if (aArr[i] > bArr[i]) return 1;
		}
		for (var i = 2; i < aArr.length && i < bArr.length; i++)
		{
			if (aArr[i] < bArr[i]) return -1;
			if (aArr[i] > bArr[i]) return 1;
		}
		if (aArr.length < bArr.length) return -1;
		if (aArr.length > bArr.length) return 1;
		var aIsDomain = a.name.search(/^[^:/?&#]+$/) > -1;
		var bIsDomain = b.name.search(/^[^:/?&#]+$/) > -1;
		if (aIsDomain && bIsDomain) return 0;
		if (aIsDomain && !bIsDomain) return -1;
		if (!aIsDomain && bIsDomain) return 1;
		if (a.name < b.name) return -1;
		if (a.name > b.name) return 1;
		return 0;
	}
}
