
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripOverlay()
{
	this.enabledObserver = null;
	this.flickerObserver = null;
	this.modeObserver = null;
	this.deepObserver = null;
	this.alwaysUseIndexObserver = null;

	this.appcontent = null;
	this.contextMenu = null;
	this.panelContainer = null;
	this.stringbundle = null;

	this.doc = null;
	this.loader = null;
	this.handler = null;
	this.highlighter = null;

	this.active = false;

	this.start = function(event)
	{
		if (event.button != 0) return;

		var doc = content.document;

		if (this.active)
		{
			this.stop(doc, false);
			return;
		}

		if (!yarip.enabled)
		{
			this.toggleEnabled(true);
			return;
		}

		this.handler = null;
		if (!doc || !doc.defaultView.yaripHandler)
		{
//			alert(this.stringbundle.getString("ERR_NO_HANDLER"));
			return;
		}
		this.handler = doc.defaultView.yaripHandler;

		if (!this.handler.start(doc))
		{
			this.stop(doc, false);
//			if (!yarip.map) alert(this.stringbundle.getString("ERR_NO_PAGES"));
//			else alert(this.stringbundle.getString("ERR_HANDLER_DIDNT_START"));
			return;
		}

		this.active = true;
		this.doc = doc;
		this.setStatus(doc, "started");
	}

	this.stop = function(doc, noStateChange)
	{
		if (!this.active || !this.handler) return false;

		if (this.handler.stop())
		{
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
			yarip.map.sortPages(false);
			yarip.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); // notify
		}
		this.active = false;

		doc = doc ? doc : this.doc;
		if (!doc) return true;

		if (!noStateChange) this.setStatus(doc, "stopped");
		this.doc = null;

		return true;
	}

	this.whitelist = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

//		xValue = prompt(this.stringbundle.getString("editForWhitelisting"), xValue + "/descendant-or-self::*");
		var obj = {
			xpath: new YaripXPath(xValue + "/descendant-or-self::*")
		}

		window.openDialog("chrome://yarip/content/whitelistdialog.xul", "whitelistdialog", "chrome,modal,resizable", obj);

		if (!obj.xpath) return;

		if (yarip.whitelist(doc, address, obj.xpath.getValue(), true, obj.xpath))
		{
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
			yarip.map.sortPages(false);
			yarip.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); // notify
		}
	}

	this.blacklist = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

		var obj = {
			xpath: new YaripXPath(xValue)
		}

		window.openDialog("chrome://yarip/content/blacklistdialog.xul", "blacklistdialog", "chrome,modal,resizable", obj);

		if (!obj.xpath) return;

		if (yarip.blacklist(doc, address, obj.xpath.getValue(), true, obj.xpath.getForce(), obj.xpath))
		{
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
			yarip.map.sortPages(false);
			yarip.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); // notify
		}
	}

	this.removeTemporarily = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

		yarip.blacklist(doc, address, xValue, false, true);
	}

	this.style = function(doc, node)
	{
		this.toggleEnabled(true);

		if (!doc || !node) return;

		var address = yarip.getAddress(doc);
		if (!address) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

		var obj = {
			xValue: xValue,
			style: node.getAttribute("style")
		}
		if (obj.style) obj.style = obj.style.replace(yarip.endOutlineRegExp, "");

		window.openDialog("chrome://yarip/content/styledialog.xul", "styledialog", "chrome,modal,resizable", obj);

//		xValue = prompt(this.stringbundle.getString("editForStyling"), xValue);
		if (!obj.xValue || obj.xValue == "") return;

//		var style = prompt(this.stringbundle.getString("editStyle"), node.getAttribute("style"));
		if (!obj.style || obj.style == "") return;

		if (yarip.style(doc, address, obj.xValue, obj.style, true))
		{
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
			yarip.map.sortPages(false);
			yarip.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); // notify
		}
	}

	this.copyXPath = function(node)
	{
		if (!node) return;

		var xValue = yarip.calculateXPath(node);
		if (!xValue) return;

		var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
		clipboardHelper.copyString(xValue);
	}

	this.startHighlighting = function(node)
	{
		if (!node) return;

		this.highlighter.doc = node.ownerDocument;
		this.highlighter.element = node;
		this.highlighter.select();
	}

	this.stopHighlighting = function()
	{
		this.highlighter.doc = null;
		this.highlighter.reset();
	}

	this.createPage = function()
	{
		this.stop();
		yarip.undoObj = {}
		yarip.toggleStyle(false);
		var address = yarip.getAddress(content.document);
		if (address && !yarip.map.obj[address]) yarip.map.add(new YaripPage(null, address));
		yarip.map.sortPages(true);
		window.openDialog("chrome://yarip/content/pagedialog.xul", "pagedialog", "chrome,resizable", address);
	}

	this.managePages = function()
	{
		this.stop();
		yarip.map.sortPages(true);
		yarip.undoObj = {}
		yarip.toggleStyle(false);
		var aAddress = yarip.getAddressArray(content.document);
		var pageAddress = null;
		for (var a in aAddress)
		{
			pageAddress = a;
			break;
		}
		window.openDialog("chrome://yarip/content/pagedialog.xul", "pagedialog", "chrome,resizable", pageAddress);
	}

	this.toggleEnabled = function(enabled, force)
	{
		this.enabledObserver.disable();
		if (!yarip.toggleEnabled(enabled) && !force)
		{
			this.enabledObserver.enable();
			return;
		}
		this.enabledObserver.enable();

		document.getElementById("yarip-enabled-menuitem").setAttribute("checked", "" + yarip.enabled);
		this.setStatus(content.document, yarip.enabled ? "enabled" : "disabled");
	}

	this.updateEnabled = function()
	{
		this.toggleEnabled(yarip.getValue(yarip.ENABLED, true, yarip.BOOLEAN), true);
	}

	this.toggleNoFlicker = function(noFlicker, force)
	{
		this.flickerObserver.disable();
		if (!yarip.toggleNoFlicker(noFlicker) && !force)
		{
			this.flickerObserver.enable();
			return;
		}
		this.flickerObserver.enable();

		document.getElementById("yarip-flicker-menuitem").setAttribute("checked", "" + yarip.noFlicker);
	}

	this.updateNoFlicker = function()
	{
		this.toggleNoFlicker(yarip.getValue(yarip.FLICKER, false, yarip.BOOLEAN), true);
	}

	this.setMode = function(mode, clear, force, update)
	{
		var address = yarip.getAddress(content.document);
		this.modeObserver.disable();
		if (!update)
		{
			if (!yarip.setMode(mode) && !force)
			{
				this.modeObserver.enable();
				return;
			}
			this.modeObserver.enable();
		}
		else this.modeObserver.enable();

		if (clear)
		{
			document.getElementById("yarip-page-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-sld-menuitem").setAttribute("checked", "false");
			document.getElementById("yarip-user-menuitem").setAttribute("checked", "false");
		}

		this.stop();
		this.toggleDeep(true/*, true*/);

		switch (mode)
		{
			case yarip.PAGE_MODE:
				document.getElementById("yarip-page-menuitem").setAttribute("checked", "true");
				break;

			case yarip.FQDN_MODE:
				document.getElementById("yarip-fqdn-menuitem").setAttribute("checked", "true");
				break;

			case yarip.SLD_MODE:
				document.getElementById("yarip-sld-menuitem").setAttribute("checked", "true");
				break;

			case yarip.USER_MODE:
				document.getElementById("yarip-user-menuitem").setAttribute("checked", "true");
				if (update) return;

				if (!address) address = yarip.address ? yarip.address : "";
				address = prompt("Edit address", address);
				if (!address || address == "")
				{
					this.setMode(yarip.FQDN_MODE, true);
					return;
				}

				yarip.address = address;
				break;

			default:
				this.setMode(yarip.FQDN_MODE, true);
		}
	}

	this.updateMode = function()
	{
		this.setMode(yarip.getValue(yarip.MODE, yarip.FQDN_MODE, yarip.INTEGER), true, true, true);
	}

	this.toggleDeep = function(deep, force)
	{
		this.deepObserver.disable();
		if (!yarip.toggleDeep(deep) && !force)
		{
			this.deepObserver.enable();
			return;
		}
		else this.deepObserver.enable();

		document.getElementById("yarip-deep-menuitem").setAttribute("checked", "" + yarip.deep);
	}

	this.updateDeep = function()
	{
		this.toggleDeep(yarip.getValue(yarip.DEEP, true, yarip.BOOLEAN), true);
	}

	this.handleEvent = function(event)
	{
		if (!window || !("yarip" in window)) return;

		switch(event.type)
		{
			case "load":
				this.loader = new YaripLoader();
				this.highlighter = new YaripHandler();

				this.enabledObserver = new YaripObserver(
					yarip.ENABLED_ROOT,
					yarip.VALUE,
					function() { yaripOverlay.updateEnabled(); }
				);
				this.flickerObserver = new YaripObserver(
					yarip.FLICKER_ROOT,
					yarip.VALUE,
					function() { yaripOverlay.updateNoFlicker(); }
				);
				this.modeObserver = new YaripObserver(
					yarip.MODE_ROOT,
					yarip.VALUE,
					function() { yaripOverlay.updateMode(); }
				);
				this.deepObserver = new YaripObserver(
					yarip.DEEP_ROOT,
					yarip.VALUE,
					function() { yaripOverlay.updateDeep(); }
				);
				this.alwaysUseIndexObserver = new YaripObserver(
					yarip.INDEX_ROOT,
					yarip.VALUE,
					function() { yarip.setAlwaysUseIndex(yarip.getValue(yarip.INDEX, false, yarip.BOOLEAN)); }
				);

				this.appcontent = document.getElementById("appcontent");
				this.contextMenu = document.getElementById("contentAreaContextMenu");
				this.panelContainer = gBrowser.mPanelContainer;
				this.stringbundle = document.getElementById("yarip-overlay-stringbundle");

				this.updateEnabled();
				this.updateNoFlicker();
//				this.updateMode();
				this.setMode(yarip.FQDN_MODE, true, true);
				this.updateDeep();

				yarip.injectCSS("chrome://yarip/content/blacklist.css", true);

				if (this.appcontent) this.appcontent.addEventListener("DOMContentLoaded", this, false);
				if (this.contextMenu) this.contextMenu.addEventListener("popupshowing", this, false);
				if (this.panelContainer) this.panelContainer.addEventListener("select", this, false);
				window.removeEventListener("load", this, false);

				break;

			case "unload":
				yarip.save();

				this.enabledObserver.unregister();
				this.flickerObserver.unregister();
				this.modeObserver.unregister();
				this.deepObserver.unregister();
				this.alwaysUseIndexObserver.unregister();

				if (this.appcontent) this.appcontent.removeEventListener("DOMContentLoaded", this, false);
				if (this.contextMenu) this.contextMenu.removeEventListener("popupshowing", this, false);
				if (this.panelContainer) this.panelContainer.removeEventListener("select", this, false);
				window.removeEventListener("load", this, false);
				window.removeEventListener("unload", this, false);
				window.removeEventListener("DOMMenuItemActive", this, false);
				window.removeEventListener("keypress", this, false);

				break;

			case "DOMContentLoaded":
				this.stop();
				this.toggleDeep(true);

				var doc = event.originalTarget;
				if (!doc || !doc.body) return;

				if (!doc.contentType.match(/^(text\/html|application\/xhtml\+xml)$/))
				{
					doc.body.setAttribute("status", "show");
					return;
				}

				var inFrame = doc.defaultView.frameElement != null;
				yarip.undoObj = {}
				var rootDoc = doc;

				// HANDLER
				if (!inFrame)
				{
					doc.defaultView.yaripHandler = new YaripHandler(null, function() { yaripOverlay.stop(); });
					if (yarip.enabled) this.setStatus(doc, "stopped");
				}
				else
				{
					while (rootDoc.defaultView.frameElement) rootDoc = rootDoc.defaultView.frameElement.ownerDocument; // get root doc
					if (rootDoc.defaultView.yaripHandler) rootDoc.defaultView.yaripHandler.addHandler(new YaripHandler(doc, function() { yaripOverlay.stop(); }));
				}

				// LOADER
				if (yarip.enabled && doc.nodeName == "#document")
				{
					if (this.loader.load(doc)) this.setStatus(rootDoc, "found");
					if (yarip.mode == yarip.USER_MODE) this.setMode(yarip.FQDN_MODE, true);
				}

				doc.body.setAttribute("status", "show");
				break;

			case "popupshowing":
				this.stop();

				var node = document.popupNode;
				var doc = node.ownerDocument;

				// YARIP-MENU 1/2
				var hasMenus = false;
				var hidden = !yarip.getAddress(doc);
				var tmpHidden = true;
				if (!hidden)
				{
					// ELEMENT-MENU
					tmpHidden = hidden || !node.localName || node.localName.search(/^(html|body)$/i) > -1;
					if (!tmpHidden)
					{
						document.getElementById("yarip-element-menu").setAttribute("label", this.stringbundle.getFormattedString("menuitemElement", [node.localName.toUpperCase()]));

						var attributesMenu = document.getElementById("yarip-element-attribute-menupopup");
						var menuitem = attributesMenu.firstChild;
						while (menuitem)
						{
							var next = menuitem.nextSibling;
							attributesMenu.removeChild(menuitem);
							menuitem = next;
						}
						var found = false;
						if (node.attributes) for (var i = 0; i < node.attributes.length; i++)
						{
							var attribute = node.attributes[i];
							if (!attribute || attribute.value == "") continue;

							switch (attribute.name)
							{
								case "status":
									if (attribute.value.match(/^(whitelisted|blacklisted|styled)$/)) continue;

								case "style":
									if (attribute.value.match(yarip.outlineRegExp)) continue;
							}

							var menuitem = document.createElement("menuitem");
							menuitem.setAttribute("label", attribute.name.toLowerCase());
							menuitem.attribute = attribute;
							menuitem.setAttribute("oncommand", "yaripOverlay.blacklist(document.popupNode.ownerDocument, this.attribute)");
							attributesMenu.insertBefore(menuitem, attributesMenu.firstChild);
							found = true;
						}
						document.getElementById("yarip-element-attribute-menu").hidden = !found;
						hasMenus = hasMenus || found;
					}
					document.getElementById("yarip-element-menu").hidden = tmpHidden;
					hasMenus = hasMenus || !tmpHidden;

					// PARENT-MENU
					tmpHidden = hidden || !node.parentNode || !node.parentNode.localName || node.parentNode.localName.search(/^(html|body)$/i) > -1;
					if (!tmpHidden)
					{
						document.getElementById("yarip-parent-menu").setAttribute("label", this.stringbundle.getFormattedString("menuitemParent", [node.parentNode.localName.toUpperCase()]));

						var attributesMenu = document.getElementById("yarip-parent-attribute-menupopup");
						var menuitem = attributesMenu.firstChild;
						while (menuitem)
						{
							var next = menuitem.nextSibling;
							attributesMenu.removeChild(menuitem);
							menuitem = next;
						}
						var found = false;
						if (node.parentNode.attributes) for (var i = 0; i < node.parentNode.attributes.length; i++)
						{
							var attribute = node.parentNode.attributes[i];
							if (!attribute || attribute.value == "") continue;

							switch (attribute.name)
							{
								case "status":
									if (attribute.value.match(/^(whitelisted|blacklisted|styled)$/)) continue;

								case "style":
									if (attribute.value.match(yarip.outlineRegExp)) continue;
							}

							var menuitem = document.createElement("menuitem");
							menuitem.setAttribute("label", attribute.name.toLowerCase());
							menuitem.attribute = attribute;
							menuitem.setAttribute("oncommand", "yaripOverlay.blacklist(document.popupNode.ownerDocument, this.attribute)");
							attributesMenu.insertBefore(menuitem, attributesMenu.firstChild);
							found = true;
						}
						document.getElementById("yarip-parent-attribute-menu").hidden = !found;
						hasMenus = hasMenus || found;
					}
					document.getElementById("yarip-parent-menu").hidden = tmpHidden;
					hasMenus = hasMenus || !tmpHidden;

					// FRAME-MENU
					tmpHidden = hidden || !gContextMenu.inFrame;
					var frameMenu = document.getElementById("yarip-frame-menu");
					if (!tmpHidden)
					{
						var frame = node.ownerDocument.defaultView.frameElement;
						frameMenu.setAttribute("label", frame.localName.toUpperCase());
						if (!yarip.getAddress(frame.ownerDocument)) frameMenu.setAttribute("disabled", true);
						else frameMenu.removeAttribute("disabled");
					}
					frameMenu.hidden = tmpHidden;
					hasMenus = hasMenus || !tmpHidden;

					// UNDO-MENU
					var undoMenu = document.getElementById("yarip-undo-menupopup");
					var menuitem = undoMenu.firstChild;
					while (menuitem && menuitem.localName != "menuseparator")
					{
						var next = menuitem.nextSibling;
						undoMenu.removeChild(menuitem);
						menuitem = next;
					}
					var found = false;
					for (var k in yarip.undoObj)
					{
						var obj = yarip.undoObj[k];
						if (!obj || obj.document != doc /*|| !obj.xValue*/) continue;

						var menuitem = document.createElement("menuitem");
						menuitem.setAttribute("label", this.stringbundle.getFormattedString("undo-" + obj.type, [obj.xValue]));
						menuitem.obj = obj;
						menuitem.setAttribute("oncommand", "yaripOverlay.undo(this.obj, true)");
						undoMenu.insertBefore(menuitem, undoMenu.firstChild);
						found = true;
					}
					document.getElementById("yarip-undo-menu-sep").hidden = !hasMenus || !found;
					document.getElementById("yarip-undo-menu").hidden = !found;
					hasMenus = hasMenus || found;
				}

				// YARIP-MENU 2/2
				document.getElementById("yarip-menu").hidden = hidden || !hasMenus;
				document.getElementById("yarip-menu-sep").hidden = hidden || !hasMenus;

				break;

			case "select":
				if (yarip.mode == yarip.USER_MODE) this.setMode(yarip.FQDN_MODE, true);
				this.setStatus(gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument);
		}
	}

	this.undoAll = function(doc)
	{
		if (!doc) return;

		for (var k in yarip.undoObj)
		{
			var obj = yarip.undoObj[k];
			if (obj && obj.document == doc /*|| !obj.xValue*/) this.undo(obj);
		}

		yarip.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); // notify
	}

	this.undo = function(obj, reload)
	{
		if (!obj) return;

//		yarip.resetOnAddress(obj.document, obj.address, obj.xValue, true);
		yarip.resetOnAddress(obj/*, true*/);
//		yarip.undoObj[obj.type + "-" + obj.xValue] = null;
		if (reload) yarip.setValue(yarip.RELOAD, (new Date()).getTime(), yarip.STRING); // notify
	}

	this.setStatus = function(doc, status)
	{
		if (!doc) return;

		if (yarip.enabled)
		{
			if (!status)
			{
				status = doc.defaultView.yaripStatus;
				switch (status)
				{
					case "enabled":
					case "stopped":
					case "found":
						break;

					case "disabled":
					case "started": if (this.active) break;
					default:
						status = "enabled";
				}
			}
		}
		else status = "disabled";

		if (doc.defaultView) doc.defaultView.yaripStatus = status;
		if (doc == gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex).contentDocument)
		{
			document.getElementById("yarip-statusbarpanel").setAttribute("status", status);
			document.getElementById("yarip-statusbarpanel").setAttribute("tooltiptext", this.stringbundle.getString("status-" + status));
		}
	}
}
