
/*
Copyright 2007-2008 Kim A. Brandt <kimabrandt@gmx.de>

This file is part of yarip.

Yarip is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

Yarip is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with yarip; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

function YaripPageDialog()
{

	this.tabSelect = function(tab, userSelect)
	{
		if (!tab) return;
		if(userSelect) this.userSelect = true;

		this.tab = tab;
//		if (this.tab != tab)
//		{
//			this.tab = tab;
//			this.refreshXPaths(tab, true);
//		}
	}

	this.setXPathValue = function(value)
	{
		if (!value) return;
		if (!this.tabs[this.tab].xpath) return;

		this.tabs[this.tab].xpath.setValue(value);
	}

	this.setStyleValue = function(value)
	{
		if (!value) return;
		if (!this.tabs[this.tab].xpath) return;

		this.tabs[this.tab].xpath.setStyle(value);
	}

	this.setDeep = function(deep, noStyle)
	{
		if (deep != true && deep != false) return;
		if (!this.tabs[this.tab].xpath) return;

		this.tabs[this.tab].xpath.setDeep(deep, noStyle);
	}

	this.setForce = function(force)
	{
		if (force != true && force != false) return;
		if (!this.tabs[this.tab].xpath) return;

		this.tabs[this.tab].xpath.setForce(force);
	}

	this.setDependsOnWhitelist = function(value)
	{
		if (value != true && value != false) return;
		if (!this.tabs[this.tab].xpath) return;
		this.tabs[this.tab].xpath.setDependsOnWhitelist(value);
	}

	this.checkXPath = function(xValue)
	{
		if (!xValue) return false;

		try { document.evaluate(xValue, document.createElement("div"), null, XPathResult.ANY_TYPE, null); }
		catch (e)
		{
//			alert(e.message);
			alert(this.stringbundle.getString("ERR_INVALID_XPATH"));
			return false;
		}

		return true;
	}

	this.addXPath = function(noStyle)
	{
		if (this.listboxPages.selectedIndex < 0) return;
		if (!this.tabs[this.tab].xpath) return;

		var xValue = this.tabs[this.tab].xpath.getValue();
		if (xValue != "" && this.checkXPath(xValue))
		{
			var xpath = this.tabs[this.tab].xpath.clone(true);
			if (!noStyle) xpath.createStyle(true);
			this.tabs[this.tab].list.add(xpath);
			this.tabs[this.tab].list.sortXPaths(true);
			this.refreshXPaths(this.tab, true);
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
		}
		else this.tabs[this.tab].textbox.select();
	}

	this.updateXPath = function(noStyle)
	{
		if (!this.tabs[this.tab].xpath || this.tabs[this.tab].tree.currentIndex < 0) return;

		var xValue = this.tabs[this.tab].xpath.getValue();
		if (xValue != "" && this.checkXPath(xValue))
		{
			var index = this.tabs[this.tab].tree.currentIndex;
			this.tabs[this.tab].list.remove(this.tabs[this.tab].aXPaths[index][0]);
			var xpath = this.tabs[this.tab].xpath;
			if (!noStyle) xpath.createStyle(true);
			this.tabs[this.tab].list.add(xpath);
			this.tabs[this.tab].list.sortXPaths(true);
			this.refreshXPaths(this.tab, true);
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
		}
		else this.tabs[this.tab].textbox.select();
	}

	this.removeXPath = function()
	{
		if (this.tabs[this.tab].tree.currentIndex < 0) return;

		var arr = [];
		var rangeCount = this.tabs[this.tab].tree.view.selection.getRangeCount();
		for (var i = 0; i < rangeCount; i++)
		{
			var start = {};
			var end = {};
			this.tabs[this.tab].tree.view.selection.getRangeAt(i, start, end);
			for(var j = start.value; j <= end.value; j++) this.tabs[this.tab].list.remove(this.tabs[this.tab].aXPaths[j][0]);
		}

		this.refreshXPaths(this.tab, true);
		yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
	}

	this.removeAllXPaths = function()
	{
		if (!this.tabs[this.tab].list || this.tabs[this.tab].list.length == 0) return;

		this.tabs[this.tab].list.reset();
		this.refreshXPaths(this.tab, true);
		yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
	}

	this.addExtPage = function()
	{
		if (this.listboxPages.selectedIndex < 0) return;
		if (!this.tabs["extension"].xpath) return;

		var id = this.tabs["extension"].xpath.getValue();
		if (id != "" && id != this.listboxPages.getItemAtIndex(this.listboxPages.selectedIndex).page.getId())
		{
			var xpath = this.tabs["extension"].xpath.clone();
			this.tabs["extension"].list.add(xpath);
			this.tabs["extension"].list.sortXPaths(true);
			this.refreshXPaths("extension", true);
			yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
		}
	}

	this.removeExtPage = function()
	{
		if (this.tabs["extension"].tree.currentIndex < 0) return;

		var arr = [];
		var rangeCount = this.tabs["extension"].tree.view.selection.getRangeCount();
		for (var i = 0; i < rangeCount; i++)
		{
			var start = {};
			var end = {};
			this.tabs["extension"].tree.view.selection.getRangeAt(i, start, end);
			for(var j = start.value; j <= end.value; j++)
			{
				var id = yarip.map.obj[this.tabs["extension"].aXPaths[j][0]].getId();
				this.tabs["extension"].list.remove(id);
			}
		}

		this.refreshXPaths("extension", true);
		yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
	}

	this.newPage = function()
	{
		this.listboxPages.selectedIndex = -1;
		this.renamePage(new YaripPage(null, "www.example.net"));
	}

	this.copyPage = function()
	{
		if (this.listboxPages.selectedIndex < 0) return;

		var page = this.listboxPages.getItemAtIndex(this.listboxPages.selectedIndex).page.clone(true);
		this.renamePage(page);
	}

	this.checkPage = function(name)
	{
		if (!name) return false;

		try { if (!name.match(yarip.uriRegExp)) throw new YaripException(this.stringbundle.getString("ERR_INVALID_PAGE")); }
		catch (e)
		{
			alert(e.message);
			return false;
		}

		return true;
	}

	this.selectTab = function(listboxItem)
	{
		if (!listboxItem) return;

		if (listboxItem.page.whitelist.length == 0)
		{
			if (listboxItem.page.blacklist.length == 0)
			{
				if (listboxItem.page.styles.length == 0)
				{
					if (listboxItem.page.extension.length == 0)
					{
						this.tabSelect("whitelist");
						document.getElementById("tabbox").selectedIndex = 0;
					}
					else
					{
						this.tabSelect("extension");
						document.getElementById("tabbox").selectedIndex = 3;
					}
				}
				else
				{
					this.tabSelect("styles");
					document.getElementById("tabbox").selectedIndex = 2;
				}
			}
			else
			{
				this.tabSelect("blacklist");
				document.getElementById("tabbox").selectedIndex = 1;
			}
		}
		else
		{
			this.tabSelect("whitelist");
			document.getElementById("tabbox").selectedIndex = 0;
		}
	}

	this.selectListboxItem = function(pageName)
	{
		if (!pageName) return null;

		for (var i = 0; i < this.listboxPages.getRowCount(); i++)
		{
			var item = this.listboxPages.getItemAtIndex(i);
			if (item.page.getName() == pageName)
			{
				this.listboxPages.selectedIndex = i;
//				this.listboxPages.ensureIndexIsVisible(this.listboxPages.getRowCount() - 1);
				this.listboxPages.ensureIndexIsVisible(i);
				return item;
			}
		}

		return null;
	}

	this.renamePage = function(page)
	{
		var oldName = null;
		if (!page)
		{
			if (this.listboxPages.selectedIndex < 0) return false;
//			page = this.listboxPages.removeItemAt(this.listboxPages.selectedIndex).page;
			page = this.listboxPages.getItemAtIndex(this.listboxPages.selectedIndex).page;
			oldName = page.getName();
		}
		var name = prompt("Enter name of page:", page.getName());
		if (!name || name == "" || !this.checkPage(name)) return false;

		if (oldName) yarip.map.removeByName(page.getName());
		page.setName(name);
		if (!yarip.map.obj[name]) yarip.map.add(page);
		else
		{
			var tmp = yarip.map.obj[page.getName()];
			for (var v in page.whitelist.xpaths) tmp.whitelist.add(page.whitelist.xpaths[v].clone());  // append/overwrite
			for (var v in page.blacklist.xpaths) tmp.blacklist.add(page.blacklist.xpaths[v].clone());
			for (var v in page.styles.xpaths) tmp.styles.add(page.styles.xpaths[v].clone());
			for (var v in page.extension.xpaths) tmp.extension.add(page.extension.xpaths[v].clone());
		}
		yarip.map.sortPages(false);
		yarip.setChangeType(yarip.ESSENTIAL_CHANGE);

		if (oldName && this.listboxPages.selectedIndex > -1) this.listboxPages.removeItemAt(this.listboxPages.selectedIndex);

		var i = 0;
		var tmpPage = new YaripPage("", name);
		while (i < this.listboxPages.getRowCount() && yarip.map.pageSort(tmpPage, this.listboxPages.getItemAtIndex(i).page) > 0) i++;

		var listitem = this.listitemDummy.cloneNode(true);
		listitem.setAttribute("label", name);
		listitem.page = page;

		if (i < this.listboxPages.getRowCount())
		{
			var sortResult = yarip.map.pageSort(tmpPage, this.listboxPages.getItemAtIndex(i).page);
			if (sortResult != 0)
			{
				if (sortResult < 0)
				{
					var oldListitem = this.listboxPages.insertItemAt(i, name, "");
					this.listboxPages.replaceChild(listitem, oldListitem);          // https://bugzilla.mozilla.org/show_bug.cgi?id=437394
				}
				else this.listboxPages.appendChild(listitem);
			}
		}
		else this.listboxPages.appendChild(listitem);

		this.listboxPages.selectedIndex = i;
		this.listboxPages.ensureIndexIsVisible(i);
		this.selectTab(listitem);
		this.refreshExtMenulist(true);

		return true;
	}

	this.removePage = function()
	{
		if (this.listboxPages.selectedIndex < 0) return;

		yarip.map.removeByName(this.listboxPages.removeItemAt(this.listboxPages.selectedIndex).page.getName());
		this.refreshPages(true, true);
		yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
	}

	this.refreshPages = function(load, dontReloadPages)
	{
		if (load && !dontReloadPages)
		{
			this.listboxPages.removeEventListener("select", this, false);
			while (this.listboxPages.getRowCount() > 1 && this.listboxPages.removeItemAt(1));
			this.listboxPages.addEventListener("select", this, false);

			for (var p in yarip.map.obj)
			{
				var page = yarip.map.obj[p];
				if (!page) continue;
//				var listitem = this.listboxPages.appendItem(p);
				var listitem = this.listitemDummy.cloneNode(true);
				listitem.setAttribute("label", p);
				listitem.page = page;
				this.listboxPages.appendChild(listitem);
			}

		}

		var page = null;
		if (load || this.listboxPages.selectedIndex < 0) page = new YaripPage("0");
		else
		{
			var listitem = this.listboxPages.getSelectedItem(0);
			if (!this.userSelect) this.selectTab(listitem); // TODO selectTab(listitem) should be optional
			page = listitem.page;
		}

		this.dialogheaderPage.setAttribute("title", page.getName());
		this.tabs["whitelist"].list = page.whitelist;
		this.tabs["blacklist"].list = page.blacklist;
		this.tabs["styles"].list = page.styles;
		this.tabs["extension"].list = page.extension;
		if (load) this.tabs["extension"].xpath = new YaripItem();

		this.refreshXPaths(null, true);
		this.refreshExtMenulist(load);
	}

	this.refreshXPaths = function(tab, load)
	{
		if (!tab)
		{
			this.refreshXPaths("whitelist", load);
			this.refreshXPaths("blacklist", load);
			this.refreshXPaths("styles", load);
			this.refreshXPaths("extension", load);
			return;
		}

		switch (tab)
		{
			case "whitelist":
			case "blacklist":
			case "styles":
				if (load)
				{
					this.tabs[tab].aXPaths = this.tabs[tab].list ? this.tabs[tab].list.getXPathsArray(tab) : [];
					this.tabs[tab].tree.view = new YaripTreeView(this.tabs[tab].aXPaths, this.tabs[tab].columns, this.tabs[tab].aXPaths.length);
				}

				if (load || this.tabs[tab].tree.currentIndex < 0)
				{
					this.tabs[tab].xpath = new YaripXPath();
					this.tabs[tab].update.hidden = true;
				}
				else
				{
					var xValue = this.tabs[tab].aXPaths[this.tabs[tab].tree.currentIndex][0];
					if (this.tabs[tab].list.xpaths[xValue]) this.tabs[tab].xpath = this.tabs[tab].list.xpaths[xValue].clone();
					this.tabs[tab].update.hidden = false;
				}

				this.tabs[tab].textbox.value = this.tabs[tab].xpath.getValue();
				this.tabs[tab].deepMenulist.selectedIndex = this.tabs[tab].xpath.getDeep() ? 0 : 1;
				if (tab == "blacklist")
				{
					this.tabs["blacklist"].forceMenulist.selectedIndex = this.tabs["blacklist"].xpath.getForce() ? 0 : 1;
					this.tabs["blacklist"].dependsOnWhitelistMenulist.selectedIndex = this.tabs["blacklist"].xpath.getDependsOnWhitelist() ? 0 : 1;
				}
				if (tab == "styles") this.tabs["styles"].styleTextbox.value = this.tabs[tab].xpath.getStyle();
				break;

			case "extension":
				if (load)
				{
					var aXPaths = [];
					if (this.tabs["extension"].list)
					{
						var arr = this.tabs["extension"].list.getXPathsArray();
						for (var i = 0; i < arr.length; i++)
						{
							var idArr = arr[i];
							var page = yarip.map.objId[idArr[0]];
							if (page) aXPaths.push([page.getName()]);
							else
							{
								this.tabs["extension"].list.remove(idArr[0]);
								yarip.setChangeType(yarip.ESSENTIAL_CHANGE);
							}
						}
					}
					this.tabs["extension"].aXPaths = aXPaths;
					this.tabs["extension"].tree.view = new YaripTreeView(this.tabs["extension"].aXPaths, this.tabs["extension"].columns, this.tabs["extension"].aXPaths.length);
				}
		}
	}

	this.reload = function()
	{
		var pageName = null;
		if (this.listboxPages.selectedIndex > -1) pageName = this.listboxPages.getSelectedItem(0).page.getName();
		this.refreshPages(true);
		if (pageName) this.selectTab(this.selectListboxItem(pageName));
	}

	this.handleEvent = function(event)
	{
		if (!event) return;

		switch(event.target.id)
		{
			case "listbox-pages":
				this.refreshPages();
				break;

			case "whitelist-tree":
			case "blacklist-tree":
			case "styles-tree":
				this.refreshXPaths(this.tab);
		}
	}

	this.initTab = function(tab)
	{
		if (!tab) return;

		switch (tab)
		{
			case "whitelist":
				this.tabs[tab] = {};
				this.tabs[tab].textbox = document.getElementById(tab + "-xpath");
				this.tabs[tab].deepMenulist = document.getElementById(tab + "-deep");
				this.tabs[tab].update = document.getElementById(tab + "-update-xpath");
				this.tabs[tab].tree = document.getElementById(tab + "-tree");
				this.tabs[tab].tree.addEventListener("select", this, false);
				this.tabs["whitelist"].columns = {
					wlXPath: 0,
					wlDeep: 1,
					wlFound: 2,
					wlNotFound: 3,
					wlWhitelisted: 4
				}
				break;

			case "blacklist":
				this.tabs[tab] = {};
				this.tabs[tab].textbox = document.getElementById(tab + "-xpath");
				this.tabs[tab].deepMenulist = document.getElementById(tab + "-deep");
				this.tabs[tab].forceMenulist = document.getElementById("blacklist-force");
				this.tabs["blacklist"].dependsOnWhitelistMenulist = document.getElementById("blacklist-dependsOnWhitelist");
				this.tabs[tab].update = document.getElementById(tab + "-update-xpath");
				this.tabs[tab].tree = document.getElementById(tab + "-tree");
				this.tabs[tab].tree.addEventListener("select", this, false);
				this.tabs["blacklist"].columns = {
					blXPath: 0,
					blDeep: 1,
					blForce: 2,
					blDoW: 3,
					blFound: 4,
					blNotFound: 5,
					blBlacklisted: 6
				}
				break;

			case "styles":
				this.tabs[tab] = {};
				this.tabs[tab].textbox = document.getElementById(tab + "-xpath");
				this.tabs["styles"].styleTextbox = document.getElementById("styles-style");
				this.tabs[tab].deepMenulist = document.getElementById(tab + "-deep");
				this.tabs[tab].update = document.getElementById(tab + "-update-xpath");
				this.tabs[tab].tree = document.getElementById(tab + "-tree");
				this.tabs[tab].tree.addEventListener("select", this, false);
				this.tabs["styles"].columns = {
					stXPath: 0,
					stStyle: 1,
					stDeep: 2,
					stFound: 3,
					stNotFound: 4
				}
				break;

			case "extension":
				this.tabs["extension"] = {};
				this.tabs["extension"].menulist = document.getElementById("extension-menulist");
				this.tabs["extension"].tree = document.getElementById("extension-tree");
				this.tabs["extension"].columns = {
					extPage: 0
				}
		}
	}

	this.refreshExtMenulist = function(load)
	{
		if (load)
		{
			var menupopup = document.createElement("menupopup");
			for (var p in yarip.map.obj)
			{
				var page = yarip.map.obj[p];
				if (!page) continue;

				var menuitem = document.createElement("menuitem");
				menuitem.setAttribute("label", p);
				menuitem.setAttribute("value", page.getId());
				menupopup.appendChild(menuitem);
			}
			this.tabs["extension"].menulist.replaceChild(menupopup, this.tabs["extension"].menulist.firstChild);
		}

//		if (this.tabs["extension"].menulist.firstChild.childNodes.length > 0)
////		if (this.tabs["extension"].menulist.itemCount > 0) // >= Firefox 3
//		{
//			this.tabs["extension"].menulist.selectedIndex = 0;
//			this.tabs["extension"].xpath.setValue(this.tabs["extension"].menulist.selectedItem.getAttribute("value"));
//		}
		if (this.tabs["extension"].menulist.firstChild.childNodes.length > 0)
		{
			if (this.tabs["extension"].menulist.selectedIndex < 0) this.tabs["extension"].menulist.selectedIndex = 0;
			this.tabs["extension"].xpath.setValue(this.tabs["extension"].menulist.selectedItem.getAttribute("value"));
		}
	}

	this.load = function()
	{
		if (!window || !("yarip" in window)) return;

		var pageName = null;
		if ("arguments" in window && window.arguments.length > 0) pageName = window.arguments[0];

		this.stringbundle = document.getElementById("page-dialog-stringbundle");
		this.listboxPages = document.getElementById("listbox-pages");
		this.listitemDummy = document.getElementById("listitem-dummy");
		this.dialogheaderPage = document.getElementById("dialogheader-page");

		this.userSelect = false;
		this.tab = "whitelist";
		this.tabs = {};
		this.initTab("whitelist");
		this.initTab("blacklist");
		this.initTab("styles");
		this.initTab("extension");

		this.reloadObserver = new YaripObserver(
			yarip.RELOAD_ROOT,
			yarip.VALUE,
			function() { try { yaripPageDialog.reload(); } catch(e) {} }
		);

//		this.listboxPages.addEventListener("select", this, false);
		this.refreshPages(true);

		if (pageName) this.selectTab(this.selectListboxItem(pageName));
	}

	this.unload = function()
	{
		this.reloadObserver.unregister();
		this.listboxPages.removeEventListener("select", this, false);
		this.tabs["whitelist"].tree.removeEventListener("select", this, false);
		this.tabs["blacklist"].tree.removeEventListener("select", this, false);
		this.tabs["styles"].tree.removeEventListener("select", this, false);
//		this.tabs["extension"].tree.removeEventListener("select", this, false);
		yarip.save();
		yarip.toggleStyle();
	}
}

function YaripTreeView(table, columns, rowcount)
{
	this.table = table ? table : null;
	this.columns = columns ? columns : null;
	this.rowCount = rowcount ? rowcount : 0;
	this.cycleHeader = function(column) {}
	this.getCellProperties = function(row, col, props) {}
	this.getCellText = function(row, col) { return this.table[row][this.columns[col.id]]; }
	this.getColumnProperties = function(colid, col, props) {}
	this.getImageSrc = function(row, col) { return null; }
	this.getLevel = function(row) { return 0; }
	this.getRowProperties = function(row, props) {}
	this.isContainer = function(row) { return false; }
	this.isSeparator = function(row) { return false; }
	this.isSorted = function(row) { return false; }
	this.setTree = function(treebox) { this.treebox = treebox; }
}
